#include "PrefsDialog.h"
//#include "SettingsInfo.h"
#include <QDir>
#include <QSqlDatabase>
#include <QSqlQuery>
#include <QFileDialog>
#include <QFileInfo>
#include <QSettings>
#include <QMenu>
#include <QDebug>

PrefsDialog::PrefsDialog(QWidget * parent, Qt::WindowFlags f, int tabShowed)
        : QDialog(parent, f)
{
    setupUi(this);

     /* translate buttons in QDialogButtonBox buttonBox */
    QPushButton * bbOkButton = buttonBox->button( QDialogButtonBox::Ok );
    QPushButton * bbCancelButton = buttonBox->button( QDialogButtonBox::Cancel );
    bbOkButton->setText( tr("OK") );
    bbCancelButton->setText( tr("Cancel") );

    /* context menu for button pbAccountRemove (which in the end has been morphed into a QToolButton) */
    deleteAccountAction = new QAction ( QIcon(":images/fileclose.png"), tr("Delete selected account : sure ?"), this);
    connect( deleteAccountAction, SIGNAL( triggered() ), this, SLOT( pbAccountRemoveClicked() ) );
    QMenu *minu = new QMenu(this);
    minu->addAction(deleteAccountAction);
    pbAccountRemove->setMenu(minu);

    connect( listWidget, SIGNAL( currentRowChanged (int ) ), this, SLOT( listWidgetChanged( int) ) );
    connect( listWidget, SIGNAL( itemDoubleClicked ( QListWidgetItem * ) ), this, SLOT( pbAccountEditClicked() ) );
    connect( pbAccountSave, SIGNAL( clicked () ), this, SLOT( pbAccountSaveClicked() ) );
    connect( pbAccountAdd, SIGNAL( clicked () ), this, SLOT( pbAccountAddClicked() ) );
    connect( pbAccountEdit, SIGNAL( clicked () ), this, SLOT( pbAccountEditClicked() ) );
    connect( pbAccountCancel, SIGNAL( clicked () ), this, SLOT( pbAccountCancelClicked() ) );
    connect( cbSound, SIGNAL( currentIndexChanged( int ) ), this, SLOT( cbSoundChanged(int) ) );
    connect( checkBoxInbox, SIGNAL( stateChanged( int ) ), this, SLOT( checkBoxInboxChanged(int) ) );
    connect( buttonBox, SIGNAL( accepted () ), this, SLOT( okAccept() ) );
    connect( buttonBox, SIGNAL( rejected () ), this, SLOT( reject() ) );

    restoreSettings();
    listWidgetUpdate();

    if (tabShowed>0)
        tabWidget->setCurrentIndex(tabShowed);
}

void PrefsDialog::okAccept()
{
    QSettings settings;//QSettings settings(settingsFile, QSettings::IniFormat);
    settings.beginGroup("MAIN");
    settings.setValue("check_interval", spCheck->value());
    settings.setValue("first_check", spFirstCheck->value());
    settings.setValue("timeout_duration", spTimeoutDuration->value());
    settings.setValue("popup_duration", spPopupDuration->value());
    settings.setValue("popups", cbPopups->currentIndex());
    settings.setValue("sound", cbSound->currentIndex());
    if ( cbSound->count() > 3 )
        settings.setValue("sound_file", cbSound->itemText(3));
    settings.endGroup();

    accept();
}

void PrefsDialog::restoreSettings()
{
     // Get Settings from config file
     QSettings settings;//QSettings settings(settingsFile, QSettings::IniFormat);
     settings.beginGroup("MAIN");
         spCheck->setValue( settings.value("check_interval", 4).toInt() );//imapCheckIntervaltTime = 15;
         spFirstCheck->setValue( settings.value("first_check", 1).toInt() );
         spPopupDuration->setValue( settings.value("popup_duration", 30).toInt() );
         spTimeoutDuration->setValue( settings.value("timeout_duration", 90).toInt() );
         int playSoundOk = settings.value("sound",  1).toInt();
         cbPopups->setCurrentIndex( settings.value("popups",  0).toInt() );
         QString soundFileOther = settings.value("sound_file", "").toString();
     settings.endGroup();

     // Apply settings
     if ( !soundFileOther.isEmpty() )
     {
         if (cbSound->count()<4)
         {
             cbSound->addItem(soundFileOther);
             //qDebug() << "GRINCHEUX";
         }
         else
             cbSound->setItemText(3,soundFileOther);
     }

     cbSound->setCurrentIndex(playSoundOk);
}

void PrefsDialog::pbAccountAddClicked()
{

         QSqlQuery query;
         query.prepare("INSERT INTO accounts (name, user, pwd, check_time, inbox) "
                       "VALUES (:name, :user, :pwd, DATETIME('NOW'), :inbox )");
         query.bindValue(":name", "New_Account");
         query.bindValue(":user", "");
         query.bindValue(":pwd", "");
         query.bindValue(":inbox", "");
         query.exec();
         //db.close();
         listWidgetUpdate( listWidget->count() );
     //}
}

void PrefsDialog::pbAccountRemoveClicked()
{

         QSqlQuery query;
         if ( query.exec("DELETE FROM accounts WHERE id=" + listWidget->currentItem()->data(Qt::UserRole).toString()) )
         {
             listWidgetUpdate( listWidget->currentRow()-1 );
         }
}

void PrefsDialog::pbAccountSaveClicked()
{

         QSqlQuery query;
         query.prepare("UPDATE accounts SET name=:name, user=:user, pwd=:pwd, inbox=:inbox "
                   "WHERE id=" + listWidget->currentItem()->data(Qt::UserRole).toString());
         query.bindValue(":name", leName->text().trimmed());
         query.bindValue(":user", leUser->text().trimmed());
         query.bindValue(":pwd", lePwd->text());
         query.exec();
          if (checkBoxInbox->isEnabled())
             query.bindValue(":inbox", leInbox->text().trimmed());
         else
             query.bindValue(":inbox", "");
         query.exec();

         toggleAccountEditMode(false);
         listWidgetUpdate(listWidget->currentRow());
}

void PrefsDialog::pbAccountEditClicked()
{
    toggleAccountEditMode(true);
}

void PrefsDialog::pbAccountCancelClicked()
{
    toggleAccountEditMode(false);
}

void PrefsDialog::toggleAccountEditMode(bool boul)
{
    listWidget->setDisabled(boul);
    pbAccountAdd->setDisabled(boul);
    pbAccountRemove->setDisabled(boul);
    pbAccountEdit->setDisabled(boul);
    buttonBox->setDisabled(boul);

    gbBasic->setEnabled(boul);
    gbMisc->setEnabled(boul);
    pbAccountSave->setEnabled(boul);
    pbAccountCancel->setEnabled(boul);
}

void PrefsDialog::listWidgetChanged(int idx)
{
    if (idx == -1) // accounts list is empty
    {
         leName->setText( "" );
         leUser->setText( "" );
         lePwd->setText( "" );
         leInbox->setText( "" );
        return;
    }

         QSqlQuery query( "SELECT id, name, user, pwd, inbox FROM accounts WHERE id="
                          + listWidget->item(idx)->data(Qt::UserRole).toString() );

         while (query.next()) {
             leName->setText( query.value(1).toString() );
             leUser->setText( query.value(2).toString() );
             lePwd->setText( query.value(3).toString() );
             leInbox->setText( query.value(4).toString() );
             deleteAccountAction->setText( tr("Delete account '%1' : sure ?").arg(query.value(1).toString()) );
         }

         query.finish();//db.close();

         if (!leInbox->text().isEmpty())
             checkBoxInbox->setChecked(true);
         else
             checkBoxInbox->setChecked(false);
     //}
}

void PrefsDialog::listWidgetUpdate(int itemIdx)
{

    listWidget->clear();


         QSqlQuery query("SELECT id, name FROM accounts");
         while (query.next()) {
             QListWidgetItem *item = new QListWidgetItem;
             item->setData( Qt::UserRole, query.value(0).toString() );
             item->setText( query.value(1).toString() );
             item->setIcon(QIcon(":images/gaim.png"));

             listWidget->addItem(item);
         }
         
		 query.finish();//db.close();

         if ( listWidget->count() < 1 ) // if imap accounts list is empty
         {
             pbAccountRemove->setDisabled(true);
             pbAccountEdit->setDisabled(true);
             gbBasic->setEnabled(false);
         }
         else
         {
             pbAccountRemove->setDisabled(false);
             pbAccountEdit->setDisabled(false);
             listWidget->setCurrentRow(itemIdx);
         }
     //}
}

void PrefsDialog::checkBoxInboxChanged(int state)
{
    bool boul=false;
    if (state>0)
        boul = true;
    leInbox->setEnabled(boul);
}

void PrefsDialog::cbSoundChanged(int cbidx)
{
    if (cbidx == 2)
    {
       QString fileName = QFileDialog::getOpenFileName(
               this,
               tr("Open Sound File"),
               QDir::homePath(),
               tr("Sound Files (*.wav)") // tr("Sound Files (*.wav *.mp3 *.ogg)")
               );

        if ( fileName.isEmpty() )
            cbSound->setCurrentIndex(1);
        else if (!QFile::exists(fileName))
            cbSound->setCurrentIndex(1);
        else
        {
            if ( cbSound->count() < 4 )
            {
                cbSound->addItem(fileName);//qDebug() << "SIMPLET";
            }
            else
                cbSound->setItemText(3,fileName);

            cbSound->setCurrentIndex(3);
        }
    }
}

